/**
 * A helper that checks equality of the given arguments.
 */
export const eq = /** @return {boolean} */ (/** @type {any} */ x, /** @type {any} */ y) => {
  if (x === y) {
    return true
  }

  const xConstructor = x?.constructor

  if (xConstructor === y?.constructor) {
    if (xConstructor === Number) {
      return (isNaN(x) && isNaN(y)) || +x === +y
    }

    if (xConstructor === Date) {
      return +x === +y
    }

    if (xConstructor === String || xConstructor === RegExp) {
      return '' + x === '' + y
    }

    if (xConstructor === Array) {
      return x.length === y.length &&
        x.every((/** @type {any} */ item, /** @type {number} */ index) => eq(item, y[index]))
    }

    if (xConstructor === Object) {
      const keysOfX = Object.keys(x)
      return keysOfX.length === Object.keys(y).length && keysOfX.every(key => eq(x[key], y[key]))
    }

    if (xConstructor === Set || xConstructor === Map) {
      if (x.size !== y.size) {
        return false
      }
      const xa = [...x]
      const ya = [...y]
      return xa.every(xv => ya.find(yv => eq(xv, yv)))
    }
  }

  return false
}

export const tests = {
  eq: (/** @type {import('bun:test').Expect} */ expect) => {
    expect(eq(true, true)).toBe(true)
    expect(eq(NaN, NaN)).toBe(true)
    expect(eq(null, undefined)).toBe(false)
    expect(eq(42, 42)).toBe(true)
    // eslint-disable-next-line no-new-wrappers
    expect(eq(42, new Number(42))).toBe(true)
    expect(eq(42, Number(42))).toBe(true)
    // eslint-disable-next-line no-new-wrappers
    expect(eq(new Number(42), Number(42))).toBe(true)
    expect(eq(42, '42')).toBe(false)
    expect(eq('42', '42')).toBe(true)
    // eslint-disable-next-line no-new-wrappers
    expect(eq('42', new String('42'))).toBe(true)
    expect(eq('42', String('42'))).toBe(true)
    // eslint-disable-next-line no-new-wrappers
    expect(eq(String('42'), new String('42'))).toBe(true)
    expect(eq(/42/, /42/)).toBe(true)
    expect(eq(/42/, /42/g)).toBe(false)
    expect(eq(new Date(42), new Date(42))).toBe(true)
    expect(eq(new Date(), new Date(42))).toBe(false)
    expect(eq({ j: '42', c: 42 }, { c: 42, j: '42' })).toBe(true)
    expect(eq([42, '42'], [42, '42'])).toBe(true)
    expect(eq(new Set(['42', 42]), new Set([42, '42']))).toBe(true)
    expect(eq(new Set(['42', 42]), new Set([42]))).toBe(false)
    expect(eq(new Set([42, undefined]), new Set([42]))).toBe(false)
    expect(eq(
      new Map([[{ j: 42 }, { J: '42' }], [{ c: 42 }, { C: '42' }]]),
      new Map([[{ c: 42 }, { C: '42' }], [{ j: 42 }, { J: '42' }]])
    )).toBe(true)
    expect(eq(
      new Map([[{ j: 42 }, { J: '42' }], [{ c: 42 }, { C: '42' }]]),
      new Map([[{ j: '42' }, { J: 42 }], [{ c: '42' }, { C: 42 }]])
    )).toBe(false)
  }
}
