import { is } from './is.js'

/**
 * A replacement for the `in` operator (not to be confused with the `for-in` loop) that works properly.
 */
export const has = (/** @type {any} */ key, /** @type {any} */ ref) =>
  (is(String, key) || is(Number, key) || is(Symbol, key)) && Object.hasOwnProperty.call(ref ?? Object, key)

export const tests = {
  has: (/** @type {import('bun:test').Expect} */ expect) => {
    const obj = { key: 'K', null: 'N' }

    expect('key' in obj).toBe(true)
    expect(has('key', obj)).toBe(true)

    expect('null' in obj).toBe(true)
    expect(has('null', obj)).toBe(true)

    // @ts-expect-error
    expect(null in obj).toBe(true)
    expect(has(null, obj)).toBe(false)

    expect('toString' in obj).toBe(true)
    expect(has('toString', obj)).toBe(false)
  },

  'has: null': (/** @type {import('bun:test').Expect} */ expect) => {
    let typeError

    try {
      // @ts-expect-error
      'key' in null
    } catch (error) {
      typeError = error
    }

    expect(typeError instanceof TypeError) // Cannot use 'in' operator to search for 'key' in null
    expect(has('key', null)).toBe(false)
  }
}
