import { is } from './is.js'

/**
 * Language translations helper.
 */
export const locale = (
  /** @type {Partial<Record<PropertyKey, Partial<Record<PropertyKey, string>>>>} */ map,
  /** @type {string} */ defaultVersion
) => (
  /** @type {string} */ text,
  /** @type {string} */ version = defaultVersion
) => {
  const textV = map?.[version]?.[text]
  const textD = map?.[defaultVersion]?.[text]

  return is(String, textV) ? textV : is(String, textD) ? textD : text
}

export const tests = {
  locale: (/** @type {import('bun:test').Expect} */ expect) => {
    const _ = locale({
      default: { Password: 'Hasło' },
      button: { Login: 'Zaloguj' }
    }, 'default')

    expect(_('Login')).toEqual('Login')
    expect(_('Password')).toEqual('Hasło')

    expect(_('Undefined text')).toEqual('Undefined text')

    expect(_('Login', 'button')).toEqual('Zaloguj')

    expect(_('Password', 'undefined_version')).toEqual('Hasło')
    expect(_('Undefined text', 'undefined_version')).toEqual('Undefined text')

    expect(_('toString')).toEqual('toString')
    expect(_('toString', 'undefined_version')).toEqual('toString')
  }
}
