/**
 * A helper that implements TypeScript’s `Pick` utility type.
 *
 * @type {<T extends Partial<Record<PropertyKey, any>>, K extends (keyof T)[]>(obj: T, keys: K) => Pick<T, K[number]>}
 */
export const pick = (/** @type {Partial<Record<PropertyKey, any>>} */ obj, /** @type {any[]} */ keys) =>
  // @ts-expect-error
  Object.fromEntries(Object.entries(obj).filter(([key]) => keys.includes(key)))

/**
 * A helper that implements TypeScript’s `Omit` utility type.
 *
 * @type {<T extends Partial<Record<PropertyKey, any>>, K extends (keyof T)[]>(obj: T, keys: K) => Omit<T, K[number]>}
 */
export const omit = (/** @type {Partial<Record<PropertyKey, any>>} */ obj, /** @type {any[]} */ keys) =>
  // @ts-expect-error
  Object.fromEntries(Object.entries(obj).filter(([key]) => !keys.includes(key)))

export const tests = {
  pick: (/** @type {import('bun:test').Expect} */ expect) => {
    const obj = { a: 42, b: '42', c: 17 }

    expect(pick(obj, ['a', 'b'])).toEqual({ a: 42, b: '42' })
  },

  omit: (/** @type {import('bun:test').Expect} */ expect) => {
    const obj = { a: 42, b: '42', c: 17 }

    expect(omit(obj, ['c'])).toEqual({ a: 42, b: '42' })
  }
}
