/**
 * A helper that protects calls to nested properties by a `Proxy` that initializes non-existent values with an empty object.
 */
// @ts-expect-error
export const pro = (/** @type {any} */ ref) => new Proxy(ref, {
  get (target, key) {
    return pro(target[key] = target[key] ?? {})
  }
})

export const tests = {
  pro: (/** @type {import('bun:test').Expect} */ expect) => {
    const ref = {}

    pro(ref).one.two[3][4] = 1234

    expect(ref).toEqual({ one: { two: { 3: { 4: 1234 } } } })

    pro(ref).one.two.tree = 123

    expect(ref).toEqual({ one: { two: { 3: { 4: 1234 }, tree: 123 } } })

    pro(ref).one.two = undefined

    expect(ref).toEqual({ one: { two: undefined } })

    delete pro(ref).one.two

    expect(ref).toEqual({ one: {} })

    pro(ref).one.two.three.four

    expect(ref).toEqual({ one: { two: { three: { four: {} } } } })

    pro(ref).one.two.three.four = 1234

    expect(ref).toEqual({ one: { two: { three: { four: 1234 } } } })
  }
}
