const ZEROS = '0'.repeat(16)
let counter = 0

/**
 * A helper that generates a UUID v1 identifier (with a creation timestamp).
 *
 * - The optional `node` parameter should have the format `/^[0123456789abcdef]+$/`.
 *   Its value will be trimmed to last 12 characters and left padded with zeros.
 */
export const uuid1 = ({
  date = new Date(),
  node = Math.random().toString(16).slice(2)
} = {}) => {
  const time = ZEROS + (10000 * (+date + 12219292800000)).toString(16)

  counter = (counter + 1) & 16383

  return time.slice(-8).concat(
    '-',
    time.slice(-12, -8),
    // @ts-expect-error
    -1,
    time.slice(-15, -12),
    '-',
    (8 | (counter >> 12)).toString(16),
    (ZEROS + (counter & 4095).toString(16)).slice(-3),
    '-',
    (ZEROS + node).slice(-12))
}

export const tests = {
  uuid1: (/** @type {import('bun:test').Expect} */ expect) => {
    for (let i = 1; i <= 22136; ++i) {
      const uuid = uuid1()

      i === 1 && expect(uuid.split('-')[3]).toEqual('8001')
      i === 4095 && expect(uuid.split('-')[3]).toEqual('8fff')
      i === 4096 && expect(uuid.split('-')[3]).toEqual('9000')
      i === 9029 && expect(uuid.split('-')[3]).toEqual('a345')
      i === 13398 && expect(uuid.split('-')[3]).toEqual('b456')
      i === 16384 && expect(uuid.split('-')[3]).toEqual('8000')
      i === 17767 && expect(uuid.split('-')[3]).toEqual('8567')
    }
  },

  'uuid1: node': (/** @type {import('bun:test').Expect} */ expect) => {
    expect(uuid1({ node: '000123456789abc' }).split('-')[4]).toEqual('123456789abc')
    expect(uuid1({ node: '123456789' }).split('-')[4]).toEqual('000123456789')
  },

  'uuid1: date': (/** @type {import('bun:test').Expect} */ expect) => {
    expect(uuid1({ date: new Date(323325000000) })).toStartWith('c1399400-9a71-11bd')
  }
}
